<?php

namespace Fastrack\Core\Model;

use DNADesign\Elemental\TopPage\FluentExtension;
use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\LinkField\Models\Link;
use SilverStripe\LinkField\Form\LinkField;
use SilverStripe\Forms\FieldList;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property string $Excerpt
 * @property int $Sort
 * @method Link CallToAction()
 * @method Gallery Gallery()
 * @method Imgix Image()
 */
class Slide extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_SLIDES';

	private static string $table_name = 'Slide';

	private static array $extensions = [
		FluentExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Excerpt' => 'Text',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'CallToAction' => Link::class,
		'Gallery' => Gallery::class,
		'Image' => Imgix::class,
	];

	private static array $owns = [
		'CallToAction',
		'Gallery',
		'Image',
	];

	private static string $default_sort = 'Sort';

	private static array $translate = [
		'Title',
		'Excerpt',
	];

	private static array $hidden_fields = [
		'GalleryID',
		'Sort',
		'CallToActionID',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$fields->addFieldsToTab('Root.Main', [
			LinkField::create('CallToAction'),
		]);
		ModelUtilities::hideFields(self::class, $fields);
		return $fields;
	}


	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage slides',
				'category' => 'Gallery',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function Thumbnail(): string
	{
		if ($this->Image()->exists()) {
			return $link = $this->Image()->Fill(100, 100);
			return "<img src=\"$link\">";
		}
		return '';
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
